@@ -267,8 +267,13 @@ sub get_arg_spec {
         return;
     }
 
+    $opt->{and} = [];
+    $opt->{or} = [];
+    $opt->{not} = [];
+
     return {
         1                   => sub { $opt->{1} = $opt->{m} = 1 },
+        'and=s'             => $opt->{and},
         'A|after-context:-1'  => sub { shift; $opt->{A} = _context_value(shift) },
         'B|before-context:-1' => sub { shift; $opt->{B} = _context_value(shift) },
         'C|context:-1'        => sub { shift; $opt->{B} = $opt->{A} = _context_value(shift) },
@@ -330,6 +335,8 @@ sub get_arg_spec {
         },
         'noignore-directory|noignore-dir=s' => _generate_ignore_dir('--noignore-dir', $opt),
         'nopager'           => sub { $opt->{pager} = undef },
+        'not=s'             => $opt->{not},
+        'or=s'              => $opt->{or},
         'passthru'          => \$opt->{passthru},
         'print0'            => \$opt->{print0},
         'p|proximate:1'     => \$opt->{p},
@@ -925,6 +932,11 @@ sub mutex_options {
             v => 1,
             'with-filename' => 1,
         },
+        and => {
+            g => 1,
+            not => 1,
+            or => 1,
+        },
         break => {
             L => 1,
             c => 1,
@@ -1000,6 +1012,7 @@ sub mutex_options {
             C => 1,
             H => 1,
             L => 1,
+            and => 1,
             break => 1,
             c => 1,
             column => 1,
@@ -1012,7 +1025,9 @@ sub mutex_options {
             l => 1,
             m => 1,
             match => 1,
+            not => 1,
             o => 1,
+            or => 1,
             output => 1,
             p => 1,
             passthru => 1,
@@ -1079,6 +1094,10 @@ sub mutex_options {
             L => 1,
             l => 1,
         },
+        not => {
+            and => 1,
+            g => 1,
+        },
         o => {
             A => 1,
             B => 1,
@@ -1096,6 +1115,10 @@ sub mutex_options {
             'show-types' => 1,
             v => 1,
         },
+        or => {
+            and => 1,
+            g => 1,
+        },
         output => {
             A => 1,
             B => 1,
